'use strict'

var plugin_version = chrome.app.getDetails().version;

// кэшированный код скриптов и стилей
let scripts = [];

// информация по возможным играм
let apps = [];

// информация по возможным к скану сервисам
let services = [];

// время последней загрузки скриптов
let script_load_time = null;

// если последяя загрузка была с ошибкой, то ошибку кэшируем
let script_load_error = null;

// последняя версия файла с предметами
let pricebase_items_version = 0;

// предметы с каким временем последнего обновления получать
let settings = {
    updated : 60,
    service : '',
    category : ''
};

loadSettings();
function loadSettings() {

    // стандартные натсройки
    let settings_prop = settings;

    if (window.localStorage["trade-assistant-settings"]) {
        try {
            settings = JSON.parse(window.localStorage["trade-assistant-settings"]);
        } catch (e) {
            console.log('Не удалось восстановить сохраненные настройки, восстановлены стандартные!');
        }
    }

    for (let prop in settings_prop) {
        if (!settings[ prop ]) {
            settings[ prop ] = settings_prop[ prop ];
        }
    }

    saveSettings();
}

function saveSettings() {
    window.localStorage["trade-assistant-settings"] = JSON.stringify(settings);
}

/**
 * Загрузка данных с сервера или с кэша
 */
function getAssistantInfo(cache_time, items, callback) {

    if (typeof callback !== 'function') {
        callback = function(){};
    }

    // если скрипты обновлять не нужно то выдать сразу
    if (script_load_time && (Date.now() - script_load_time) <= cache_time) {
        if (script_load_error) {

            return callback({
                success : false,
                error : script_load_error
            });

        } else {

            return callback({
                success : true,
                scripts : scripts,
                apps : apps,
                services : services,
                settings : settings,
                prices : items ? pricebase_last_scan_prices : undefined,
                info : items ? pricebase_last_scan_info : undefined
            });

        }

    }

    // нужно обновить скрипты перед выдачей
    api.request({
        url : '/trade-assistant/info',
        data : {
            v : plugin_version
        },
        method : 'GET'
    }, function (err, res, body) {

        script_load_time = Date.now();

        if (err || !body) {
            let message = "";

            if (body && (body.message || body.code)) {
                message = body.message || body.code; // CODY ФОРМИРОВАТЬ В СООБЩЕНИЕ
            }

            script_load_error = message || 'Unknown error';

            // нет подписки или какая-то другая ошибка
            return callback({
                success : false,
                error : script_load_error
            });
        }

        script_load_error = null;

        if (!pricebase_items_version) {
            pricebase_items_version = body.pricebase_items_version;
            // загрузим предметы
            loadItemsFile(false);
        }

        // подписка и ссылки есть
        scripts = body.sites;
        apps = body.apps;
        services = body.services;

        return callback({
            success : true,
            scripts : scripts,
            apps : apps,
            services : services,
            settings : settings,
            prices : items ? pricebase_last_scan_prices : undefined,
            info : items ? pricebase_last_scan_info : undefined
        });

    });
}
// первая загрузка данных после инициализации
getAssistantInfo();


chrome.runtime.onMessage.addListener(function(request, sender, callback) {
    // запрос на получение всей информации
    if (request.type === 'assistant/get-info') {
        getAssistantInfo(request.cache_time, request.items, callback);
        return true;
    }

    if (request.type === 'assistant/load-prices') {
        loadPrices(request.service, request.category, request.domain, callback);
        return true;
    }

    if (request.type === 'assistant/load-favourite') {
        loadFavourite(callback);
        return true;
    }

    if (request.type === 'assistant/add-favourite') {
        addFavourite(request.item, callback);
        return true;
    }

    if (request.type === 'assistant/remove-favourite') {
        removeFavourite(request.item, callback);
        return true;
    }

    if (request.type == 'assistant/execute-script') {
        updateAndExecute(request.tab.id, request.tab, 60 * 1000);
        return callback();
    }

    if (request.type == 'settings/save') {
        settings[ request.option ] = request.value;
        saveSettings();
        return callback();
    }


});

chrome.tabs.onUpdated.addListener(function (tabID, info, tab) {

    // если какая-то страница была загружена
    if (info.status === 'complete') {
        // обновить скрипты за последние 10 минут
        updateAndExecute(tabID, tab, 10 * 60 * 1000);

    }

});

function updateAndExecute(tabID, tab, time) {
    getAssistantInfo(time, null, function () {

        let url = document.createElement('a');
        url.href = tab.url;

        // проверим есть ли скрипт для сайта
        let site_script = scripts.find(e => e.domain == url.hostname),
            icon_path = {};

        // прописываем иконпки всех размеров
        [16, 24, 32, 48, 96].forEach(function (size) {
            icon_path[size] = 'icons/' + (site_script ? 'green' : 'red') + size + '.png';
        });

        chrome.browserAction.setIcon({
            path: icon_path,
            tabId : tabID
        });

        if (site_script) {

            chrome.permissions.contains({
                origins: [`${url.protocol}//${url.hostname}/`]
            }, function(result) {

                if (result) {
                    executeScript(tabID, site_script.assets);
                }

            });

        }

    });
}

function executeScript(tabID, links) {

    if (links.css) {

        api.request({
            url : links.css,
            prefix: false,
            method : 'GET'
        }, function (err, res, body) {
            if (err) return;

            chrome.tabs.insertCSS(tabID, {
                'code' : body
            }, function() {});

        });

    }

    if (links.js) {

        api.request({
            url : links.js,
            prefix: false,
            method : 'GET'
        }, function (err, res, body) {
            if (err) return;

            let series_functions = [];

            let to_execute = [
                {
                    type : 'file',
                    style : true,
                    value : "/assets/css/fontawesome.css"
                },
                {
                    type : 'file',
                    value : "/assets/vendor/jquery.js"
                },
                {
                    type : 'file',
                    value : "/assets/vendor/api.js"
                },
                {
                    type : 'code',
                    value : body
                },
            ];

            for (let i = 0; i < to_execute.length; i++) {

                series_functions.push(function(callback) {

                    if (to_execute[i].style) {

                        chrome.tabs.insertCSS(tabID, {
                            [to_execute[i].type] : to_execute[i].value
                        }, function() {
                            callback();
                        });

                    } else {

                        chrome.tabs.executeScript(tabID, {
                            [to_execute[i].type] : to_execute[i].value
                        }, function() {
                            callback();
                        });

                    }

                });

            }

            async.series(series_functions, function(err, results) {});

        });

    }


}

